#include "smasher.h"

Smasher::Smasher(Object *parent, bool *move_up, bool *move_down, bool *turn_right, bool *turn_left, bool *smash, float move_speed, float turn_speed) : Material(parent) {
	this->move_up = move_up;
	this->move_down = move_down;
	//this->move_right = move_right;
	//this->move_left = move_left;
	this->turn_right = turn_right;
	this->turn_left = turn_left;
	this->smash = smash;
	this->move_speed = move_speed / 5.0f;
	this->turn_speed = turn_speed / 5.0f;

	forward = new MultiAtomDir();
	sideways = new Direction();
	body = new Roster(this);
	glove = new Roster(this);
	spring = new Stretch(this);

	extension = 1.0f;
	ext_speed = 0.2f;
	ext_dir = 0;
}

void Smasher::stepFunc() {
	Material::stepFunc();
	const float max_ext = 2.5f;

	forward->calc();
	sideways->dx = forward->dy;
	sideways->dy = -forward->dx;

	if(ext_dir != 0.0f) {
		extension += ext_dir;
		if(extension <= 1.0f) {
			extension = 1.0f;
			ext_dir = 0.0f;
			//glove->applyForce(forward, ext_speed * 100.0f);
		}
		else if(extension >= max_ext) {
			extension = max_ext;
			ext_dir = 0.0f;
			//glove->applyForce(forward, -ext_speed * 100.0f);
		}
		spring->stretch(extension);
		if(ext_dir > 0) {
			glove->applyMove(forward, ext_speed * 60.0f);
		}
		else if(ext_dir < 0) {			
			glove->applyMove(forward, -ext_speed * 60.0f);
		}

	}
	if(extension >= max_ext && !*smash) {
		ext_dir = -ext_speed;
	}
	else if(extension <= 1.0f && *smash) {
		ext_dir = ext_speed;
	}

	/*
	if(*move_up)
		body->applyForce(forward, move_speed);
	if(*move_down)
		body->applyForce(forward, -move_speed);
	if(*move_right)
		body->applyForce(sideways, move_speed);
	if(*move_left)
		body->applyForce(sideways, -move_speed);
		*/
	if(*move_up)
		applySpeed(forward, move_speed);
	if(*move_down)
		applySpeed(forward, -move_speed);
	//if(*move_right)
	//	applySpeed(sideways, move_speed * 0.5f);
	//if(*move_left)
	//	applySpeed(sideways, -move_speed * 0.5f);

	/*
	if(*move_up)
		body->applyForce(0.0f, move_speed);
	if(*move_down)
		body->applyForce(0.0f, -move_speed);
	if(*move_right)
		body->applyForce(move_speed, 0.0f);
	if(*move_left)
		body->applyForce(-move_speed, 0.0f);
	*/
	if(*turn_right) {
		glove->applySpeed(sideways, turn_speed);
		body->applySpeed(sideways, -turn_speed * 0.3f);
		//body->applyForce(forward, -turn_speed * 1.0f);
	}
	if(*turn_left) {
		glove->applySpeed(sideways, -turn_speed);
		body->applySpeed(sideways, turn_speed * 0.3f);
		//body->applyForce(forward, turn_speed * 1.0f);
	}
}

void Smasher::overrideUpdateLink(Link *l) {
	float mult;
	float dx, dy;
	Atom *a = l->a;
	Atom *b = l->b;

	dx = a->x - b->x;
	dy = a->y - b->y;

	mult = float(l->rl / sqrt(dx * dx + dy * dy));
	if(mult < l->lbp || mult > l->ubp) {
		delete l;
		return;
	}
	if(mult > 100) mult = 100.0f;

	float comp = 1.0f / (a->rm + b->rm);
	float forcex = l->k * comp * (dx * mult - dx);
	float forcey = l->k * comp * (dy * mult - dy);

	float m1, m2;

	if(a->in(body) && !b->in(body)) {
		m1 = 0.5f;
		m2 = 1.5f;
	}
	else if(!a->in(body) && b->in(body)) {
		m2 = 0.5f;
		m1 = 1.5f;
	}
	else {
		m1 = 1.0f;
		m2 = 1.0f;
	}

	a->fx += m1 * forcex;
	a->fy += m1 * forcey;

	b->fx -= m2 * forcex;
	b->fy -= m2 * forcey;

	Material::overrideUpdateLink(l);
}

Atom *Smasher::placeAtom(Atom *a) {
	if(a->group == 1) {
		return body->placeAtom(a);
	}
	else if(a->group == 2) {
		forward->addBack(a);
		return body->placeAtom(a);
	}
	else if(a->group == 3) {
		return glove->placeAtom(a);
	}
	else if(a->group == 4) {
		forward->addFront(a);
		return glove->placeAtom(a);
	}
	
	return Material::placeAtom(a);
}

Link *Smasher::placeLink(Link *l) {
	if(l->a->in(body) != l->b->in(body)) {
		l->mode = LINK_MODE_OVERRIDE;
	}

	if(l->group == 1) {
		return spring->placeLink(l);
	}
	
	return Material::placeLink(l);
}
